/* global gcforms_builder, wpf */
'use strict';

var GCFormsBuilderApprovals = window.GCFormsBuilderApprovals || ( function( document, window, $ ) {

	/**
	 * Public functions and properties.
	 *
	 * @since 1.0.0
	 *
	 * @type {object}
	 */
	var app = {

		/**
		 * jQuery selector for holder.
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		$holder: $( '.gcforms-panel-content-section-approvals' ),

		/**
		 * Start the engine.
		 *
		 * @since 1.0.0
		 */
		init: function() {

			// Do that when DOM is ready.
			$( document ).ready( app.ready );
		},

		/**
		 * DOM is fully loaded.
		 *
		 * @since 1.0.0
		 */
		ready: function() {

			app.events();
		},

		/**
		 * Register JS events.
		 *
		 * @since 1.0.0
		 */
		events: function() {

			$( '#gcforms-builder' )
				.on( 'gcformsSaved', app.requiredFields.init )
				.on( 'gcformsSettingsBlockAdded', app.approvalBlockAdded )
				// .on( 'gcformsStatusMapTableAddedRow', app.statusMapTableAddRow )
				.on( 'change.gcformsApprovals', '#gcforms-panel-field-settings-approvals_enable', app.approvalsToggle )
				.on( 'input.gcformsApprovals', '.gcforms-status-map-table .http-key-source', app.updateNameAttr ) // 自定义审批状态

			// app.$holder
			// 	.on( 'change.gcformsApprovals', '.gcforms-field-map-table .gcforms-field-map-select', app.changeSourceSelect )
			// 	.on( 'change.gcformsApprovals', '.gcforms-field-map-table .gcforms-field-map-is-secure-checkbox', app.changeSecure )
			// 	.on( 'click.gcformsApprovals', '.gcforms-field-map-table .gcforms-field-map-custom-value-close', app.closeCustomValue );

				// 审批状态，删除行
				.on('click', '.gcforms-status-map-table .remove', function(e) {
					e.preventDefault();
					app.statusMapTableDeleteRow(e, $(this));
				})

				// 审批状态，添加行
				.on('click', '.gcforms-status-map-table .add', function(e) {
					e.preventDefault();
					app.statusMapTableAddRow(e, $(this));
				});
		},

		/**
		 * Status map table - Delete row.
		 *
		 * @since 1.2.0
		 * @since 1.6.1.2 Registered `gcformsStatusMapTableDeletedRow` trigger.
		 */
		statusMapTableDeleteRow: function( e, el ) {

			var $this  = $( el ),
				$row   = $this.closest( 'tr' ),
				$table = $this.closest( 'table' ),
				$block = $row.closest( '.gcforms-builder-settings-block' ),
				total  = $table.find( 'tr' ).length;

			if ( total > '1' ) {
				$row.remove();

				$( '#gcforms-builder' ).trigger( 'gcformsStatusMapTableDeletedRow', [ $block ] );
			}
		},

		/**
		 * Status map table - Add row.
		 *
		 * @since 1.2.0
		 * @since 1.6.1.2 Registered `gcformsStatusMapTableAddedRow` trigger.
		 */
		statusMapTableAddRow: function( e, el ) {

			var $this  = $( el ),
				$row   = $this.closest( 'tr' ),
				$block = $row.closest( '.gcforms-builder-settings-block' ),
				choice = $row.clone().insertAfter( $row );

			choice.find( 'input' ).val( '' );
			choice.find( 'select :selected' ).prop( 'selected', false );
			choice.find( '.key-destination' ).attr( 'name', '' );

			$( '#gcforms-builder' ).trigger( 'gcformsStatusMapTableAddedRow', [ $block, choice ] );
		},

		/**
		 * 添加新审批时重置状态。
		 *
		 * @since {VERSION}
		 *
		 * @param {object} event  Event object.
		 * @param {object} $block New Approval block.
		 */
		approvalBlockAdded: function( event, $block ) {

			if ( ! $block.length || 'approval' !== $block.data( 'block-type' ) ) {
				return;
			}

			$block.find( '.gcforms-status-map-table .gcforms-status-map-custom-value-close' ).trigger( 'click' );
			var $table = $block.find( '.gcforms-status-map-table' );

			// 清除自定义审批状态的表字段.
			$block.find( 'tr:not(:first-child)' ).remove();
			// 清除自定义审批状态的表单内容。
			$block.find( '.gcforms-status-map-table input' ).attr( 'value', '' );

		},

		/**
		 * Resetting statuss when we add a new table row for mapping.
		 *
		 * @since {VERSION}
		 *
		 * @param {object} event   Event object.
		 * @param {object} $block  jQuery selector on Approval block.
		 * @param {object} $choice jQuery selector on new table row.
		 */
		// statusMapTableAddRow: function( event, $block, $choice ) {

		// 	if ( ! $block.length || 'approval' !== $block.data( 'block-type' ) || ! $choice.length ) {
		// 		return;
		// 	}

		// 	$choice.find( '.gcforms-status-map-custom-value-close' ).trigger( 'click' );
		// },

		/**
		 * Toggle the displaying approval settings depending on if the
		 * approvals are enabled.
		 *
		 * @since 1.0.0
		 */
		approvalsToggle: function() {

			app.$holder
				.find( '.gcforms-builder-settings-block-approval, .gcforms-approvals-add' )
				.toggleClass( 'hidden', '0' === $( this ).val() );
		},

		/**
		 * Status map table, update key source.
		 *
		 * @since 1.0.0
		 */
		updateNameAttr: function() {

			var $this    = $( this ),
				value    = $this.val(),
				$row     = $this.closest( 'tr' ),
				$targets = $row.find( '.gcforms-status-map-custom-value' ),
				name     = $targets.data( 'name' );

			if ( ! value && '' !== value ) {
				return;
			}

			// if ( $row.find( 'td.status' ).hasClass( 'status-is-custom-value' ) ) {
				$targets = $row.find( '.gcforms-status-map-custom-value' );
			// }

			$targets.each( function( idx, status ) {

				var newName = name + $( status ).data( 'suffix' );

				// Allow characters (lowercase and uppercase), numbers, decimal point, underscore and minus.
				$( status ).attr( 'name', newName.replace( '{source}', value.replace( /[^a-zA-Z0-9._-]/gi, '' ) ) );
			} );
		},

		/**
		 * Event-callback when the source select was changed on "添加自定义值".
		 *
		 * @since {VERSION}
		 */
		// changeSourceSelect: function() {

		// 	var $row          = $( this ).closest( 'tr' );
		// 		isCustomValue = ( this.value && this.value === 'custom_value' );

		// 	if ( isCustomValue ) {
		// 		$( this ).attr( 'name', '' );
		// 		$row.find( 'td.status' ).toggleClass( 'status-is-custom-value', isCustomValue );
		// 		$row.find( '.http-key-source' ).trigger( 'input.gcformsApprovals' );
		// 	}
		// },

		/**
		 * Change a "自定义值" input type - `text` to `password` or vice versa.
		 *
		 * @param {object} event Event object.
		 */
		// changeSecure: function( event ) {

		// 	var $row = $( this ).closest( 'tr' );

		// 	$row.find( '.gcforms-status-map-custom-value' ).attr( 'type', event.target.checked ? 'password' : 'text' );
		// },

		/**
		 * Event-callback when the close button for "自定义值" was clicked.
		 *
		 * @since {VERSION}
		 *
		 * @param {object} event Event object.
		 */
		// closeCustomValue: function( event ) {

		// 	var $row = $( this ).closest( 'tr' );

		// 	event.preventDefault();

		// 	$row.find( 'td.status' ).removeClass( 'status-is-custom-value' );
		// 	$row.find( '.gcforms-status-map-select' ).prop( 'selectedIndex', 0 );
		// 	$row.find( '.gcforms-status-map-is-secure-checkbox' )
		// 		.attr( 'name', '' )
		// 		.prop( 'checked', false )
		// 		.prop( 'disabled', false );
		// 	$row.find( '.gcforms-status-map-custom-value' )
		// 		.attr( 'name', '' )
		// 		.attr( 'type', 'text' )
		// 		.prop( 'readonly', false )
		// 		.val( '' );
		// },

		/**
		 * On form save notify the user about "Required statuss".
		 *
		 * @since 1.0.0
		 *
		 * @type {object}
		 */
		requiredFields: {

			/**
			 * True if we have not filled required statuss.
			 *
			 * @since 1.0.0
			 *
			 * @type {boolean}
			 */
			hasErrors: false,

			/**
			 * We need to notify the user only once.
			 *
			 * @since 1.0.0
			 *
			 * @type {boolean}
			 */
			isNotified: false,

			/**
			 * Initialization for required statuss checking.
			 *
			 * @since 1.0.0
			 */
			init: function() {

				var $settingBlocks = app.$holder.find( '.gcforms-builder-settings-block-approval' );

				if (
					! $settingBlocks.length ||
					$settingBlocks.hasClass( 'hidden' )
				) {
					return;
				}

				app.requiredFields.isNotified = false;
				$settingBlocks.each( app.requiredFields.check );
			},

			/**
			 * 进行必填字段检查。
			 *
			 * @since 1.0.0
			 */
			check: function() {

				app.requiredFields.hasErrors = false;

				$( this ).find( 'input.gcforms-required, select.gcforms-required' ).each( function() {

					var $field = $( this ),
						value  = $field.val();

					if ( _.isEmpty( value ) ) {
						$field.addClass( 'gcforms-error' );
						app.requiredFields.hasErrors = true;

					} else {
						$field.removeClass( 'gcforms-error' );
					}
				} );

				// Notify user.
				app.requiredFields.notify();
			},

			/**
			 * Modal that use for user notification.
			 *
			 * @since 1.0.0
			 */
			notify: function() {

				if ( app.requiredFields.hasErrors && ! app.requiredFields.isNotified ) {
					$.alert( {
						title: gcforms_builder.heads_up,
						content: gcforms_builder.approval_required_flds,
						icon: 'fa fa-exclamation-circle',
						type: 'orange',
						buttons: {
							confirm: {
								text: gcforms_builder.ok,
								btnClass: 'btn-confirm',
								keys: [ 'enter' ],
							},
						},
					} );

					// Save that we have already showed the user.
					app.requiredFields.isNotified = true;
				}
			},
		},
	};

	// Provide access to public functions/properties.
	return app;

}( document, window, jQuery ) );

GCFormsBuilderApprovals.init();
