<?php
/**
 * Plugin Name: 表单-审批流
 * Plugin URI:  https://gechiui.com
 * Description: 与表单插件生成的表单，增加审批流的功能支持
 * Author:      北京格尺科技
 * Author URI:  https://gechiui.com
 * Version:     1.0.2
 * Text Domain: gcforms-approvals
 * Domain Path: languages
 *
 * 产品基于开源许可协议, 协议详情 <https://www.gnu.org/licenses/>.
 *
 * @since     1.0.0
 * @author    北京格尺科技
 * @package   GCFormsApprovals
 * @license   GPL-2.0+
 * @copyright 版权所有 (c) 2023, 北京格尺科技有限公司
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if(!get_pro_license_valid()){
	add_action( 'admin_init', 'gcforms_approvals_deactivation' );
	add_action( 'admin_notices', 'gcforms_approvals_fail_gcforms_version' );
}

/**
 * 检查插件启用要求
 *
 * @since 1.0.0
 */
function gcforms_approvals_required() {

	// Load the translation files.
	load_plugin_textdomain( 'gcforms-approvals', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

	if ( version_compare( PHP_VERSION, '5.6', '<' ) ) {
		add_action( 'admin_init', 'gcforms_approvals_deactivation' );
		add_action( 'admin_notices', 'gcforms_approvals_fail_php_version' );
	} elseif (
		! function_exists( 'gcforms' ) ||
		! gcforms()->pro ||
		version_compare( gcforms()->version, '1.6.1.2', '<' )
	) {
		add_action( 'admin_init', 'gcforms_approvals_deactivation' );
		add_action( 'admin_notices', 'gcforms_approvals_fail_gcforms_version' );
	} else {
		gcforms_approvals();
	}
}
add_action( 'plugins_loaded', 'gcforms_approvals_required' );

/**
 * 停用插件。
 *
 * @since 1.0.0
 */
function gcforms_approvals_deactivation() {

	deactivate_plugins( plugin_basename( __FILE__ ) );
}

/**
 * PHP最低版本的管理通知
 *
 * @since 1.0.0
 */
function gcforms_approvals_fail_php_version() {

	echo '<div class="notice notice-error"><p>';
	esc_html_e( '表单-审批流插件已被停用。您的网站正在运行一个过时的PHP版本，该版本不再受支持，并且与Approvals插件不兼容。 ', 'gcforms-approvals' );
	echo '</p></div>';

	// phpcs:disable
	if ( isset( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
	// phpcs:enable
}

/**
 * 最低GCForms版本的管理通知
 *
 * @since 1.0.0
 */
function gcforms_approvals_fail_gcforms_version() {

	echo '<div class="notice notice-error"><p>';
	esc_html_e( '表单-审批流插件已被停用，请升级为GeChiUI专业版。', 'gcforms-approvals' );
	echo '</p></div>';

	// phpcs:disable
	if ( isset( $_GET['activate'] ) ) {
		unset( $_GET['activate'] );
	}
	// phpcs:enable
}

/**
 * 插件常量
 */
define( 'GCFORMS_APPROVALS_VERSION', '1.0.0' );
define( 'GCFORMS_APPROVALS_FILE', __FILE__ );
define( 'GCFORMS_APPROVALS_PATH', plugin_dir_path( GCFORMS_APPROVALS_FILE ) );
define( 'GCFORMS_APPROVALS_URL', plugin_dir_url( GCFORMS_APPROVALS_FILE ) );

/**
 * 获取实例类 `\GCFormsApprovals\Plugin`.
 * 此函数对于快速获取整个插件中使用的数据非常有用
 *
 * @since 1.0.0
 *
 * @return \GCFormsApprovals\Plugin
 */
function gcforms_approvals() {

	// 自动加载Approvals插件
	require_once __DIR__ . '/vendor/autoload.php';

	return \GCFormsApprovals\Plugin::get_instance();
}
